import * as fs from 'fs';
import * as dotenv_defaults from 'dotenv-defaults';
import * as LIVR from 'livr';
import * as livrExtraRules from 'livr-extra-rules';

function replace(template: string, vars: { [key: string]: string }): string {
    return template.replace(/\{\{(.+?)\}\}/g, (match: string, p1: string) => {
        // eslint-disable-next-line no-prototype-builtins
        if (vars.hasOwnProperty(p1)) {
            return vars[p1];
        }
        throw new Error(`Variable "${p1}" not set!`);
    });
}

interface DbConfigType {
    host: string;
    user: string;
    password: string;
    database: string;
    driver: string;
    port: number;
}
export interface ConfigType {
    database_config_file: string;
    db: DbConfigType;
    enable: boolean;
    log: boolean;
    sqs: {
        region: string;
    };
    queues: {
        unresponsive_node: string[];
        anchor_reboot: string[];
        site_disconnected: string[];
        battery_level: string[];
    };
}

function validateConfig(config: ConfigType, livrSchemaPath: string): ConfigType {
    const livrRules: any = JSON.parse(fs.readFileSync(livrSchemaPath).toString());
    const validator: any = new LIVR.Validator(livrRules, true);
    validator.registerRules(livrExtraRules);

    const validConfig: any = validator.validate(config);

    if (!validConfig) {
        const error: any = {
            FAILED_CONFIG: config,
            ERRORS: validator.getErrors(),
        };
        throw new Error(JSON.stringify(error, null, 2));
    }

    return validConfig;
}

function readMariaDbConfig(dbConfigPath: string): DbConfigType {
    if (fs.existsSync(dbConfigPath)) {
        const databaseJsonString: string = fs.readFileSync(dbConfigPath, 'utf8');
        const databaseJson: { [key: string]: DbConfigType } = JSON.parse(databaseJsonString);
        const NODE_ENV: string = (process.env.NODE_ENV || 'prod').toLowerCase().trim();
        return {
            host: databaseJson[NODE_ENV].host,
            port: databaseJson[NODE_ENV].port,
            user: databaseJson[NODE_ENV].user,
            password: databaseJson[NODE_ENV].password,
            database: databaseJson[NODE_ENV].database,
            driver: databaseJson[NODE_ENV].driver,
        };
    }
}

export default function confme(configPath: string, livrSchemaPath: string, options: any ): ConfigType {
    dotenv_defaults.config(options);
    const template: string = fs.readFileSync(configPath).toString();
    const configStr: string = replace(template, process.env);

    let config: ConfigType | any = {};

    try {
        config = JSON.parse(configStr);
    } catch (error) {
        console.error('CANNOT PARSE JSON:', configStr);
        throw error;
    }

    if (livrSchemaPath) {
        config = validateConfig(config, livrSchemaPath);
    }
    config.db = readMariaDbConfig(config.database_config_file);
    Object.keys(config.queues).forEach((key: string) => {
        config.queues[key] = config.queues[key].split(';');
    });
    return config;
}
